@@ -2,10 +2,11 @@ from __future__ import absolute_import
 import logging
 
 from kuyruk.kuyruk import Kuyruk
-from kuyruk.task import Task
 from kuyruk.worker import Worker
-from kuyruk.queue import Queue
-from kuyruk.version import __version__
+from kuyruk.task import Task
+from kuyruk.config import Config
+
+__version__ = '0.12.0'
 
 try:
     # not available in python 2.6
