@@ -1,37 +1,35 @@
 from __future__ import absolute_import
 import os
 import sys
+import errno
 import signal
 import socket
 import logging
+import tempfile
 import itertools
-import multiprocessing
+import subprocess
 from time import time, sleep
 from setproctitle import setproctitle
-from kuyruk.worker import Worker
-from kuyruk.version import __version__
+from kuyruk import __version__
 from kuyruk.process import KuyrukProcess
 
 logger = logging.getLogger(__name__)
 
 
 class Master(KuyrukProcess):
-    """
-    Master worker implementation that coordinates queue workers.
+    """Spawns multiple worker and supervises them by reading the list of queues
+    from configuration.
 
     """
     def __init__(self, config):
         super(Master, self).__init__(config)
         self.workers = []
-        self.override_queues = None
-        if self.config.MAX_LOAD is None:
-            self.config.MAX_LOAD = multiprocessing.cpu_count()
 
     def run(self):
         super(Master, self).run()
         setproctitle('kuyruk: master')
-        self.start_workers()
         self.maybe_start_manager_thread()
+        self.start_workers()
         self.wait_for_workers()
         logger.debug('End run master')
 
@@ -41,22 +39,21 @@ class Master(KuyrukProcess):
         queues = parse_queues_str(queues)
         logger.info('Starting to work on queues: %s', queues)
         for queue in queues:
-            worker = Worker(queue, self.config)
-            worker.start()
-            self.workers.append(worker)
+            self.spawn_new_worker(queue)
 
     def get_queues(self):
         """Return queues string."""
-        if self.override_queues:
-            return self.override_queues
+        if isinstance(self.config.QUEUES, basestring):
+            return self.config.QUEUES
 
         hostname = socket.gethostname()
         try:
-            return self.config.WORKERS[hostname]
+            return self.config.QUEUES[hostname]
         except KeyError:
-            logger.warning('No queues specified for host %r. '
-                           'Listening on default queue: "kuyruk"', hostname)
-            return 'kuyruk'
+            logger.warning(
+                'No queues specified for host %r. '
+                'Listening on default queues.', hostname)
+            return 'kuyruk, kuyruk.%s' % hostname
 
     def stop_workers(self, kill=False):
         """Send stop signal to all workers."""
@@ -77,7 +74,7 @@ class Master(KuyrukProcess):
                     any_alive = True
                 elif not self.shutdown_pending.is_set():
                     worker.kill_pg()
-                    self.spawn_new_worker(worker)
+                    self.respawn_worker(worker)
                     any_alive = True
                 else:
                     self.workers.remove(worker)
@@ -87,17 +84,20 @@ class Master(KuyrukProcess):
                              "%i seconds passed" % (time() - start))
                 sleep(1)
 
-    def spawn_new_worker(self, worker):
+    def respawn_worker(self, worker):
         """Spawn a new process with parameters same as the old worker."""
         logger.debug("Spawning new worker")
-        new_worker = Worker(worker.queue_name, self.config)
-        new_worker.start()
-        self.workers.append(new_worker)
+        self.spawn_new_worker(worker.queue)
         self.workers.remove(worker)
         logger.debug(self.workers)
 
+    def spawn_new_worker(self, queue):
+        worker = WorkerProcess(self.config, queue)
+        worker.start()
+        self.workers.append(worker)
+
     def register_signals(self):
-        signal.signal(signal.SIGINT, self.handle_sigint)
+        super(Master, self).register_signals()
         signal.signal(signal.SIGTERM, self.handle_sigterm)
         signal.signal(signal.SIGQUIT, self.handle_sigquit)
         signal.signal(signal.SIGABRT, self.handle_sigabrt)
@@ -114,18 +114,10 @@ class Master(KuyrukProcess):
         logger.warning("Handling SIGABRT")
         self.abort()
 
-    def handle_sigint(self, signum, frame):
-        logger.warning("Handling SIGINT")
-        if sys.stdin.isatty() and not self.shutdown_pending.is_set():
-            self.warm_shutdown()
-        else:
-            self.cold_shutdown()
-        logger.debug("Handled SIGINT")
-
-    def warm_shutdown(self):
-        logger.warning("Warm shutdown")
-        self.shutdown_pending.set()
-        self.stop_workers()
+    def warm_shutdown(self, sigint=False):
+        super(Master, self).warm_shutdown(sigint)
+        if not sigint:
+            self.stop_workers()
 
     def cold_shutdown(self):
         logger.warning("Cold shutdown")
@@ -150,6 +142,49 @@ class Master(KuyrukProcess):
         }
 
 
+class WorkerProcess(object):
+
+    def __init__(self, config, queue):
+        self.config = config
+        self.queue = queue
+        self.popen = None
+
+    def start(self):
+        """Runs the worker by starting another Python interpreter."""
+        # Pass configuration values to the worker
+        fd, path = tempfile.mkstemp()
+        self.config.export(fd)
+
+        command = [
+            sys.executable, '-u', '-m', 'kuyruk.__main__',
+            '--config', path,
+            '--delete-config',
+            'worker',
+            '--queue', self.queue
+        ]
+        self.popen = subprocess.Popen(
+            command, stdout=sys.stdout, stderr=sys.stderr, bufsize=1)
+
+    @property
+    def pid(self):
+        return self.popen.pid
+
+    def is_alive(self):
+        self.popen.poll()
+        return self.popen.returncode is None
+
+    def kill_pg(self):
+        """Kills the process with their children. Does not raise exception
+        if the process is not alive.
+
+        """
+        try:
+            os.killpg(self.pid, signal.SIGKILL)
+        except OSError as e:
+            if e.errno != errno.ESRCH:  # No such process
+                raise
+
+
 def parse_queues_str(s):
     """Parse command line queues string.
 
@@ -157,11 +192,10 @@ def parse_queues_str(s):
     :return: list of queue names
     """
     queues = (q.strip() for q in s.split(','))
-    queues = itertools.chain.from_iterable(parse_count(q) for q in queues)
-    return [parse_local(q) for q in queues]
+    return list(itertools.chain.from_iterable(expand_count(q) for q in queues))
 
 
-def parse_count(q):
+def expand_count(q):
     parts = q.split('*', 1)
     if len(parts) > 1:
         try:
@@ -169,9 +203,3 @@ def parse_count(q):
         except ValueError:
             return int(parts[1]) * [parts[0]]
     return [parts[0]]
-
-
-def parse_local(q):
-    if q.startswith('@'):
-        return "%s_%s" % (q[1:], socket.gethostname())
-    return q
