@@ -1,6 +1,6 @@
 from __future__ import absolute_import
 import os
-import json
+import errno
 import socket
 import logging
 import traceback
@@ -12,6 +12,7 @@ import pika.exceptions
 
 from kuyruk.message import Message
 from kuyruk.channel import LazyChannel
+from kuyruk.helpers import synchronized
 
 logger = logging.getLogger(__name__)
 
@@ -42,13 +43,6 @@ def require_declare(f):
     return inner
 
 
-def synchronized(f):
-    def inner(self, *args, **kw):
-        with self.lock:
-            return f(self, *args, **kw)
-    return inner
-
-
 class Queue(object):
 
     def __init__(self, name, channel=None, local=False):
@@ -63,7 +57,7 @@ class Queue(object):
         self.lock = RLock()
 
         if self.local:
-            self.name = "%s_%s" % (self.name, socket.gethostname())
+            self.name = "%s.%s" % (self.name, socket.gethostname())
 
     @synchronized
     def __len__(self):
@@ -94,7 +88,7 @@ class Queue(object):
         return self.channel.basic_publish(
             exchange='',
             routing_key=self.name,
-            body=json.dumps(obj),
+            body=Message.encode(obj),
             properties=properties)
 
     @synchronized
@@ -129,6 +123,7 @@ class Queue(object):
 
     @synchronized
     def delete(self):
+        """Deletes queue. Does not raise exception if queue is not found."""
         logger.warning('Deleting queue')
         try:
             return self.channel.queue_delete(queue=self.name)
@@ -153,7 +148,7 @@ class Queue(object):
             self.channel.connection.process_data_events()
         except Exception as e:
             logger.debug(e)
-            if e.args[0] == 4:  # Interrupted system call
+            if e.args[0] == errno.EINTR:  # Interrupted system call
                 # Happens when a signal is received. No harm.
                 pass
             else:
