@@ -4,6 +4,7 @@ import socket
 import signal
 import logging
 import traceback
+import multiprocessing
 from time import sleep
 from setproctitle import setproctitle
 from kuyruk import importer
@@ -12,30 +13,52 @@ from kuyruk.channel import LazyChannel
 from kuyruk.process import KuyrukProcess
 from kuyruk.helpers import start_daemon_thread
 from kuyruk.consumer import Consumer
+from kuyruk.exceptions import Reject, ObjectNotFound, Timeout
+
+try:
+    import raven
+except ImportError:
+    raven = None
 
 logger = logging.getLogger(__name__)
 
 
 class Worker(KuyrukProcess):
+    """Consumes messages from a queue and runs tasks.
 
-    def __init__(self, queue_name, config):
-        """
-        :param queue_name: Queue name that this worker gets the messages from
-        :param config: Configuration object
-        """
+    :param queue_name: The queue name to work on
+    :param config: A :class:`~kuyurk.config.Config` object
+
+    """
+    def __init__(self, config, queue_name):
         super(Worker, self).__init__(config)
         self.channel = LazyChannel.from_config(config)
-        self.queue_name = queue_name
         is_local = queue_name.startswith('@')
+        queue_name = queue_name.lstrip('@')
+        self.queue_name = queue_name
         self.queue = Queue(queue_name, self.channel, local=is_local)
         self.consumer = Consumer(self.queue)
         self.working = False
+        self.daemon_threads = [
+            self.watch_master,
+            self.watch_load,
+            self.shutdown_timer,
+        ]
+        if self.config.MAX_LOAD is None:
+            self.config.MAX_LOAD = multiprocessing.cpu_count()
+
+        if self.config.SENTRY_DSN:
+            if raven is None:
+                raise ImportError('Cannot import raven. Please install it with '
+                                  '"pip install raven".')
+            self.sentry = raven.Client(self.config.SENTRY_DSN)
+        else:
+            self.sentry = None
 
     def run(self):
-        """Run worker until stop flag is set.
-
-        Since Worker class is derived from multiprocessing.Process,
-        it will be invoked when worker.start() is called.
+        """Runs the worker and opens a connection to RabbitMQ.
+        After connection is opened, starts consuming messages.
+        Consuming is cancelled if an external signal is received.
 
         """
         super(Worker, self).run()
@@ -44,50 +67,66 @@ class Worker(KuyrukProcess):
         self.channel.basic_qos(prefetch_count=1)
         self.channel.tx_select()
         self.import_modules()
-
-        # Start threads
-        start_daemon_thread(self.watch_master)
-        start_daemon_thread(self.watch_load)
+        self.start_daemon_threads()
         self.maybe_start_manager_thread()
-        if self.config.MAX_RUN_TIME > 0:
-            start_daemon_thread(self.shutdown_timer)
+        self.consume_messages()
+        logger.debug("End run worker")
 
-        # Consume messages
+    def consume_messages(self):
+        """Consumes messages from the queue and run tasks until
+        consumer is cancelled via a signal or another thread.
+
+        """
         with self.consumer.consume() as messages:
             for message in messages:
                 self.working = True
-                self.process_task(message)
+                self.process_message(message)
                 self.channel.tx_commit()
+                logger.debug("Committed transaction")
                 self.working = False
 
-        logger.debug("End run worker")
+    def start_daemon_threads(self):
+        """Start the function as threads listed in self.daemon_thread."""
+        for f in self.daemon_threads:
+            start_daemon_thread(f)
 
     def import_modules(self):
+        """Import modules defined in the configuration.
+        This method is called before start consuming messages.
+
+        """
         for module in self.config.IMPORTS:
-            importer.import_task_module(module, self.config.IMPORT_PATH)
-
-    def process_task(self, message):
-        from kuyruk import Kuyruk
+            importer.import_module(module, self.config.IMPORT_PATH)
 
+    def process_message(self, message):
+        """Processes the message received from the queue."""
         task_description = message.get_object()
         try:
-            self.import_and_call_task(task_description)
-        # sleep() calls below prevent cpu burning
-        except Kuyruk.Reject:
+            self.apply_task(task_description)
+        except Reject:
             logger.warning('Task is rejected')
-            sleep(1)
+            sleep(1)  # Prevent cpu burning
             message.reject()
+        except ObjectNotFound:
+            self.handle_not_found(message, task_description)
+        except Timeout:
+            self.handle_timeout(message, task_description)
         except Exception:
-            logger.error('Task raised an exception')
-            logger.error(traceback.format_exc())
-            sleep(1)
+            if self.sentry:
+                ident = self.sentry.get_ident(self.sentry.captureException(
+                    extra={'task_description': task_description}))
+                logger.error("Exception caught; reference is %s", ident)
             self.handle_exception(message, task_description)
         else:
             logger.info('Task is successful')
             message.ack()
+
         logger.debug("Processing task is finished")
 
     def handle_exception(self, message, task_description):
+        """Handles the exception while processing the message."""
+        logger.error('Task raised an exception')
+        logger.error(traceback.format_exc())
         retry_count = task_description.get('retry', 0)
         if retry_count:
             logger.debug('Retry count: %s', retry_count)
@@ -100,44 +139,62 @@ class Worker(KuyrukProcess):
             if self.config.SAVE_FAILED_TASKS:
                 self.save_failed_task(task_description)
 
+    def handle_not_found(self, message, task_description):
+        """Called if the task is class task but the object with the given id
+        is not found. The default action is logging the error and acking
+        the message.
+
+        """
+        logger.error(
+            "<%s.%s id=%r> is not found",
+            task_description['module'],
+            task_description['class'],
+            task_description['args'][0])
+        message.ack()
+
+    def handle_timeout(self, message, task_description):
+        """Called when the task is timed out while running the wrapped
+        function.
+
+        """
+        logger.error('Task has timed out.')
+        self.handle_exception(message, task_description)
+
     def save_failed_task(self, task_description):
+        """Saves the task to ``kuyruk_failed`` queue. Failed tasks can be
+        investigated later and requeued with ``kuyruk reuqueue`` command.
+
+        """
         logger.info('Saving failed task')
         task_description['queue'] = self.queue_name
         failed_queue = Queue('kuyruk_failed', self.channel)
         failed_queue.send(task_description)
         logger.debug('Saved')
 
-    def import_and_call_task(self, task_description):
-        """Call task function.
-        This is the method where user modules are loaded.
+    def apply_task(self, task_description):
+        """Imports and runs the wrapped function in task."""
+        task = self.import_task(task_description)
+        args, kwargs = task_description['args'], task_description['kwargs']
 
-        """
-        module, function, cls, object_id, args, kwargs = (
-            task_description['module'],
-            task_description['function'],
-            task_description['class'],
-            task_description['object_id'],
-            task_description['args'],
-            task_description['kwargs'])
-        task, cls = importer.import_task(
-            module, cls, function, self.config.IMPORT_PATH)
-
-        # Fetch object and prepent to args if class task
-        if cls:
-            obj = cls.get(object_id)
+        # Fetch the object if class task
+        if task.cls:
+            obj = task.cls.get(args[0])
             if not obj:
-                logger.warning("<%s.%s id=%r> is not found",
-                               module, cls.__name__, object_id)
-                return
+                raise ObjectNotFound
+            args[0] = obj
 
-            args = list(args)
-            args.insert(0, obj)
-
-        logger.debug('Task %r will be executed with args=%s and kwargs=%s',
-                     task, args, kwargs)
         result = task.apply(args, kwargs)
         logger.debug('Result: %r', result)
 
+    def import_task(self, task_description):
+        """This is the method where user modules are loaded."""
+        module, function, cls = (
+            task_description['module'],
+            task_description['function'],
+            task_description['class'])
+        return importer.import_task(
+            module, cls, function, self.config.IMPORT_PATH)
+
     def is_master_alive(self):
         try:
             os.kill(os.getppid(), 0)
@@ -163,29 +220,28 @@ class Worker(KuyrukProcess):
             sleep(1)
 
     def shutdown_timer(self):
-        """Counts down from MAX_RUN_TIME. When it reaches zero sutdown
+        """Counts down from MAX_WORKER_RUN_TIME. When it reaches zero sutdown
         gracefully.
 
         """
-        sleep(self.config.MAX_RUN_TIME)
-        logger.warning('Run time reached zero, cancelling consume.')
-        self.warm_shutdown()
+        seconds = self.config.MAX_WORKER_RUN_TIME
+        if seconds > 0:
+            sleep(seconds)
+            logger.warning('Run time reached zero, cancelling consume.')
+            self.warm_shutdown()
 
     def register_signals(self):
-        # SIGINT is ignored because when pressed Ctrl-C
-        # it is sent to both master and workers.
-        signal.signal(signal.SIGINT, signal.SIG_IGN)
+        super(Worker, self).register_signals()
         signal.signal(signal.SIGTERM, self.handle_sigterm)
-        signal.signal(signal.SIGUSR1, print_stack)  # for debugging
 
     def handle_sigterm(self, signum, frame):
+        """Initiates a warm shutdown."""
         logger.warning("Catched SIGTERM")
         self.warm_shutdown()
 
-    def warm_shutdown(self):
+    def warm_shutdown(self, sigint=False):
         """Shutdown gracefully."""
-        logger.warning("Shutting down worker gracefully")
-        self.shutdown_pending.set()
+        super(Worker, self).warm_shutdown(sigint)
         self.consumer.stop()
 
     def get_stats(self):
@@ -205,9 +261,3 @@ class Worker(KuyrukProcess):
                 'consumers': method.consumer_count,
             }
         }
-
-
-def print_stack(sig, frame):
-    print '=' * 70
-    print ''.join(traceback.format_stack())
-    print '-' * 70
